function [upperbound,lowerbound] = example1(baseFileName,fileScp,fileMat,run)
% this function sets up the environment variables need for bundle and
% convscp and find upper and lower bounds for the supplied scp.
%
% scp = set covering problem
% INPUTS:
% baseFileName  - file name use for saving the output from bundle
%                 is added as a suffix to the files output from bundle
%                 is useful when trying different heuristics on one 
%                   set of output from bundle
% fileScp       - file name of the scp in standard format
%               to be passed to the bundle software
% fileMat       - file name of the scp in Matlab .mat format
% run           - 1 = run bundle to generate new output
%                 0 = use existing output
% OUTPUTS:
% upperbound    - a crude upper bound for the scp 
% lowerbound    - the lower bound from the langrange dual problem

clear u cover costs; % make sure leftovers aren't re-used

if run == 1 % if set, run bundle software
            % if not, use existing files
            
    tStart=cputime;    
    % set system environment variables in the matlab environment
    setenv('ILOG_LICENSE_FILE', '/chalmers/sw/sup/cplex-10.1/ilm/access.ilm');
    setenv('MATLAB','/chalmers/sw/sup/matlab-2006b');
    setenv('LD_LIBRARY_PATH',...
        '$MATLAB/bin/sol2:$MATLAB/sys/os/sol2:$MATLAB/bin/glnx86:$MATLAB/sys/os/glnx86');
    
    run = ['./bundle',' ',fileScp];
    [status, result] = system(run); % run bundle software on fileScp
    copyfile('uVector',[baseFileName 'uVector'],'f');
    copyfile('xVector',[baseFileName 'xVector'],'f');
    copyfile('objValue',[baseFileName 'objValue'],'f');    
disp(['cpu time(sec) running bundle: '  num2str(cputime-tStart )]); % display elapsed time
end 

tStart=cputime; 

load(fileMat); % load scp problem  from .mat file
cost = sparse(cost); % so operations with sparse vects and matrices work faster

% load the dual vector
u=load([baseFileName 'uVector']);
u=u(:,2); % remove line numbering
u=sparse(u);

% load solution from the relaxed problem 
xrelax = load([baseFileName 'xVector']);
xrelax = xrelax(:,2); % remove line numbering
xrelax = sparse(xrelax);

% load dual object value
dualobj = load([baseFileName 'objValue']);
lowerbound = dualobj;

disp(['cpu time(sec) loading data: '  num2str(cputime-tStart )]); % display elapsed time
tStart=cputime; 

%%% 
%%% find a primal solution to get an upper bound 
%%%

% get the dimenions of the cover
[m, n] = size(cover);

% quickly generate a cover by 
% adding columns until feasible
B=sparse(m,1); x=sparse(1,n);
rcost = cost./sum(cover,1); % costs / number each col covers
				%the 'r' in 'rcost' does NOT stand for the reduced costs
				
maxrcost = max(rcost)*max(sum(cover,1)); 

while nnz(B)<size(B,1) % covered yet?
    [minben,position] = min(rcost); % extract position of min rcost
    x(position)=1; % add to solution
    B=B+cover(:,position); % add to cover
    rcost(position) = maxrcost; % replace current min so it won't be found again
end

disp(['cpu time(sec) running the heuristic: '  num2str(cputime-tStart )]); % display elapsed time

upperbound = sum(cost(find(x))); % add of the costs of the columns chosen
upperbound = full(upperbound(1,1)); % want a scalar, not a 1x1 sparse matrix

disp(['upper bound: ' num2str(upperbound) '    lower bound: ' num2str(lowerbound)]);
